#!/usr/bin/env python3

#  Copyright (C) 2023 ASTRON (Netherlands Institute for Radio Astronomy)
#  SPDX-License-Identifier: GPL-3.0-or-later

"""
Remove generated source documentation files except for index.rst

If a source file is created, documentation is generated and the source file is later
removed. The documentation for this source file will persists.

This file ensures generated source documentation files, which are automatically
generated from source, are removed between every build.
"""

import os

file_dir = os.path.dirname(os.path.realpath(__file__))

clean_dir = os.path.join(file_dir, "source", "source_documentation")
print(f"Cleaning.. {clean_dir}/*")

if not os.path.exists(clean_dir):
    exit()

for file_name in os.listdir(clean_dir):
    file = os.path.join(clean_dir, file_name)

    if file_name == "index.rst":
        continue

    print(f"Removing.. {file}")
    os.remove(file)



#  Copyright (C) 2012 ASTRON (Netherlands Institute for Radio Astronomy)
#  SPDX-License-Identifier: GPL-3.0-or-later

"""Drop-in replacement for LCS/pyparameterset, which in turn wraps LCS/Common/src/ParameterSet.cc.

This replacement provides a narrow implementation. It writes compatible parsets, but cannot read
all the syntax variants supported by pyparameterset. Those variants are thus deprecated.

Not supported are:

* Vector expansion, f.e. foo=['bar'*4] and foo=[1..5],
* Most value interpretation, including booleans,
* Multi-line values,
* String escaping beyond either " or ' in the string,
* Modellng of ParameterValue as a separate class,
* Case-insensitivity,
* Tracking unused keys.

"""


class parameterset(dict):
    def __init__(self, content: dict = {}):
        self.update(content)

    def __str__(self):
        def encode(v):
            if isinstance(v, str):
                return f"'{v}'" if '"' in v else f'"{v}"'
            return str(v)

        return "\n".join(
            sorted([f"{key}={encode(value)}" for key, value in self.items()])
        )

    def add(self, key, value):
        self[key] = value

    def dict(self) -> dict:
        return self

    def isDefined(self, key) -> bool:
        return key in self

    def makeSubset(self, prefix: str):
        """
        Creates a Subset from the current ParameterSetImpl containing all the
        parameters that start with the given baseKey.
        The baseKey is cut off from the Keynames in the created subset, the
        optional prefix is put before the keynames.
        """
        return parameterset(
            {
                key[len(prefix) :]: value
                for key, value in self.items()
                if key.startswith(prefix)
            }
        )

    def fullModuleName(self, module: str):
        """
        Searches for a key ending in the given 'shortkey' and returns it full name.
        e.g: a.b.c.d.param=xxxx --> fullModuleName(d)      --> a.b.c.d
        e.g: a.b.c.d.param=xxxx --> fullModuleName(b.c)    --> a.b.c
        e.g: a.b.c.d.param=xxxx --> fullModuleName(d.param)-->
        """
        for key in self:
            if key == module:
                return key
            if key.startswith(f"{module}."):
                return module
            if key.endswith(f".{module}"):
                return key
            if (pos := key.find(f".{module}.")) >= 0:
                return key[: pos + len(module) + 1]

        return ""

    @staticmethod
    def fromString(parset: str):
        """
        Create a parameterset from a string of key=value pairs.
        """
        result = parameterset()
        result._importString(parset)
        return result

    def _importString(self, parset: str):
        lines = parset.split("\n")
        for line in lines:
            # strip comments
            if "#" in line:
                line, _ = line.split("#", 1)

            # ignore empty lines
            line = line.strip()
            if not line:
                continue

            # parse single-line key=value pairs
            assert "=" in line

            key, value = line.split("=", 1)
            key = key.strip()
            value = value.strip()

            self[key] = value

    def adoptArgv(self, argv: list[str]):
        """
        Add keys as provided on the command line in the form
        of "key=value".
        """
        for arg in argv:
            kv = arg.split("=", 1)
            if len(kv) == 2:
                self[kv[0]] = kv[1]

    def getString(self, key: str, default: str | None = None) -> str:
        """
        Return a value as string, with the given default if it is not
        found. If no default is provided either, a KeyError
        is thrown.
        """
        return self.get(key, default) if default is not None else self[key]

    def _getVector(self, key: str) -> list[str]:
        """
        Return the values of a vector, encoded as [a, b, c] or
        with the values quoted.
        """
        value = self[key]

        # value is "[element, element, element]"
        assert value[0] == "["
        assert value[-1] == "]"

        values = value[1:-1]
        elements = values.split(",")
        return [e.strip() for e in elements]

    def getIntVector(self, key: str, expandable: bool = True) -> list[int]:
        """
        Return a parset value as an integer list.

        expandable: deprecated, will be ignored.
        """
        return [int(x) for x in self._getVector(key)]

    def getDoubleVector(self, key: str, expandable: bool = True) -> list[float]:
        """
        Return a parset value as a float list.

        expandable: deprecated, will be ignored.
        """
        return [float(x) for x in self._getVector(key)]

    def getStringVector(self, key: str, expandable: bool = True) -> list[str]:
        """
        Return a parset value as a string list.

        expandable: deprecated, will be ignored.
        """

        def parse_str(s):
            if s[0] == '"' and s[-1] == '"':
                # strip double quotes
                return s[1:-1]
            elif s[0] == "'" and s[-1] == "'":
                # strip single quotes
                return s[1:-1]
            else:
                return s

        return [parse_str(x) for x in self._getVector(key)]



from unittest import TestCase

from lofar_parameterset.parameterset import parameterset


class TestParameterSet(TestCase):
    def test_construction(self):
        p = parameterset({"foo": "bar"})

        self.assertIn("foo", p)
        self.assertEqual(p["foo"], "bar")

    def test_fromString(self):
        PARSET = """
        foo = bar
        a.b.c = 3
        """

        p = parameterset.fromString(PARSET)

        self.assertIn("foo", p)
        self.assertEqual(p["foo"], "bar")
        self.assertIn("a.b.c", p)
        self.assertIn(p["a.b.c"], "3")

    def test_str(self):
        p = parameterset({"foo": "bar", "a.b.c": 3})
        self.assertEqual("\n".join(["a.b.c=3", 'foo="bar"']), str(p))

    def test_str_quoted(self):
        p = parameterset({"single_quote": "'single'", "double_quote": '"double"'})
        self.assertEqual(
            "\n".join(["double_quote='\"double\"'", "single_quote=\"'single'\""]),
            str(p),
        )

    def test_makeSubset(self):
        PARSET = """
        one.a = 1
        one.b = 2
        two.a = 3
        two.b = 4
        two.c.d = 5
        """

        p = parameterset.fromString(PARSET)
        subset = p.makeSubset("one.")

        self.assertDictEqual({"a": "1", "b": "2"}, subset)

    def test_fullModuleName(self):
        p = parameterset({"a.b.c.d": "xxxx"})

        self.assertEqual("a.b.c.d", p.fullModuleName("d"))
        self.assertEqual("a.b.c", p.fullModuleName("b.c"))
        self.assertEqual("", p.fullModuleName("d.param"))

    def test_getString(self):
        p = parameterset({"foo": "bar"})

        self.assertEqual("bar", p.getString("foo"))
        self.assertEqual("baz", p.getString("foo2", "baz"))

        with self.assertRaises(KeyError):
            _ = p.getString("baz")

    def test_getIntVector(self):
        PARSET = """
        key=[1, 2, 3, 4]
        """

        p = parameterset.fromString(PARSET)

        self.assertListEqual([1, 2, 3, 4], p.getIntVector("key"))

    def test_getStringVector(self):
        PARSET = """
        quoted=['Xre','Xim','Yre','Yim']
        unquoted=[Xre, Xim, Yre, Yim]
        """

        p = parameterset.fromString(PARSET)

        self.assertListEqual(["Xre", "Xim", "Yre", "Yim"], p.getStringVector("quoted"))
        self.assertListEqual(
            ["Xre", "Xim", "Yre", "Yim"], p.getStringVector("unquoted")
        )



#  Copyright (C) 2023 ASTRON (Netherlands Institute for Radio Astronomy)
#  SPDX-License-Identifier: GPL-3.0-or-later

import os

from lofar_parameterset._version import __version__

# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinxcontrib.apidoc",
    "sphinx_rtd_theme",
    "myst_parser"
]

# Assumes tox is used to call sphinx-build
project_root_directory = os.getcwd()

apidoc_module_dir = "../../lofar_parameterset"
apidoc_output_dir = "source_documentation"
apidoc_excluded_paths = []
apidoc_separate_modules = True
apidoc_toc_file = False
# This should include private methods but does not work
# https://github.com/sphinx-contrib/apidoc/issues/14
apidoc_extra_args = ["--private"]

# The suffix of source filenames.
source_suffix = [".rst"]

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "LOFAR Python ParameterSet"
copyright = "2023, ASTRON"

# openstackdocstheme options
repository_name = "https://git.astron.nl/lofar2.0/lofar_pyparameterset"
bug_project = "none"
bug_tag = ""
html_last_updated_fmt = "%Y-%m-%d %H:%M"

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

version = __version__

modindex_common_prefix = ["lofar_parameterset."]

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
# html_theme_path = ["."]
html_theme = "sphinx_rtd_theme"
html_static_path = ["static"]
html_css_files = [
    "css/custom.css",
]

# Output file base name for HTML help builder.
htmlhelp_basename = "%sdoc" % project

# Conf.py variables exported to sphinx rst files access using |NAME|
variables_to_export = [
    "project",
    "copyright",
    "version",
]

# Write to rst_epilog to export `variables_to_export` extract using `locals()`
# https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-rst_epilog
frozen_locals = dict(locals())
rst_epilog = "\n".join(
    map(
        lambda x: f".. |{x}| replace:: {frozen_locals[x]}",  # noqa: F821
        variables_to_export,
    )
)
# Pep is not able to determine that frozen_locals always exists so noqa
del frozen_locals
